package refund

import (
	"context"
	"crypto/rsa"
	"errors"
	"fmt"
	"git.168cad.top/go/wechat-client-sdk/util"
	"github.com/wechatpay-apiv3/wechatpay-go/core"
	"github.com/wechatpay-apiv3/wechatpay-go/core/option"
	"github.com/wechatpay-apiv3/wechatpay-go/services/refunddomestic"
	"log"
	"math/big"
	"strings"
)

type WechatRefundClient struct {
	SpMchId       *string         `json:"spMchId,omitempty"`       // 服务商商户号(如果服务商模式，该值需要填写)
	MchId         *string         `json:"mchId,omitempty"`         // 收款商户号/子商户号(如果是服务商模式，配置特约商户号)
	MchSerialNo   *string         `json:"mchSerialNo,omitempty"`   // 收款商户证书序号
	MchPrivateKey *string         `json:"mchPrivateKey,omitempty"` // 收款商户私钥
	ApiKey        *string         `json:"apiKey,omitempty"`        // apiKey
	wxClient      *core.Client    //不需要外界传入
	privateKey    *rsa.PrivateKey //不需要外界传入
}

// ApplyRefund 申请退款
func (r *WechatRefundClient) ApplyRefund(param *ApplyParam) *refunddomestic.Refund {
	wxClient := r.getWxRefundClient()
	svc := refunddomestic.RefundsApiService{Client: wxClient}
	payMoney, b := new(big.Int).SetString(new(big.Float).Mul(new(big.Float).SetFloat64(param.PayMoney), new(big.Float).SetFloat64(100)).String(), 10)
	if !b {
		panic(errors.New("金额错误请检查，请检查"))
	}
	refundMoney, b1 := new(big.Int).SetString(new(big.Float).Mul(new(big.Float).SetFloat64(param.RefundMoney), new(big.Float).SetFloat64(100)).String(), 10)
	if !b1 {
		panic(errors.New("金额错误请检查，请检查"))
	}
	resp, _, err := svc.Create(context.Background(),
		refunddomestic.CreateRequest{
			SubMchid:    r.MchId,
			OutTradeNo:  core.String(param.OutTradeNo),
			OutRefundNo: core.String(param.OutRefundNo),
			Reason:      core.String(param.Reason),
			NotifyUrl:   core.String(param.NotifyUrl),
			Amount: &refunddomestic.AmountReq{
				Currency: core.String("CNY"),
				Refund:   core.Int64(refundMoney.Int64()),
				Total:    core.Int64(payMoney.Int64()),
			},
		},
	)
	if err != nil {
		panic(errors.New(getErrMessage(err.Error())))
	}
	log.Println(fmt.Sprintf("微信退款调用结果：%s", resp.String()))
	return resp
}

func (r *WechatRefundClient) getWxRefundClient() *core.Client {
	if r.wxClient != nil {
		return r.wxClient
	}
	// 使用 utils 提供的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
	if r.MchPrivateKey == nil {
		panic(errors.New("商户私钥未配置，请检查"))
	}
	mchPrivateKey, err := util.LoadPrivateKey(*r.MchPrivateKey)
	if err != nil {
		panic(errors.New("商户私钥配置错误，请检查"))
	}
	r.privateKey = mchPrivateKey

	// 使用商户私钥等初始化 client，并使它具有自动定时获取微信支付平台证书的能力
	opts := []core.ClientOption{
		option.WithWechatPayAutoAuthCipher(*r.SpMchId, *r.MchSerialNo, mchPrivateKey, *r.ApiKey),
	}
	wc, err1 := core.NewClient(context.Background(), opts...)
	if err1 != nil {
		panic(err1)
	}
	r.wxClient = wc
	return wc
}

func getErrMessage(msg string) string {
	startStr := "response:["
	endStr := "Header"
	startIndex := strings.Index(msg, startStr)
	endIndex := strings.Index(msg, endStr)
	rs := msg
	if startIndex >= 0 && endIndex > 0 {
		rs = msg[startIndex+len(startStr) : endIndex]
	}
	return strings.ReplaceAll(rs, "\n", " ")
}
